require('mootools');

var fs = require('fs');

var winston  = require('winston');
winston.remove(winston.transports.Console);
winston.add(winston.transports.Console, { colorize: true });
var logger = winston;

var async = require('async');
var redis = require("redis");
var pty = require('pty.js');

exports.CreateServer = function(options) {
	return new NodeRedis(options);
}

var NodeRedis = new Class({
	Implements: [Options, Events],
	options: {
		redis: {
			folder: 'redis',
			debug: false,
			config: {
				port: 6379,
				bind: '127.0.0.1'
			}
		},
		web: {
			port: 6380
		}
	},
	
	initialize: function(options) {
		var self = this;
		
		// Setup options
		self.setOptions(options);
		
		// Start redis
		async.series([
			function(callback) {
				self.initRedisConfig(callback);
			},
			function(callback) {
				self.startRedis(callback);
			},
			function(callback) {
				self.createRedisClient(callback);
			},
			function(callback) {
				self.startRedisWeb(callback);
			}
		],
		function(err) {
			if(err) logger.error(err);
			else {
				self.fireEvent('started');
			}
		});
		
		// Stop redis
		process.on('exit', function () {
			console.log('TEST !');
			
			self.stop();
		});
	},
	
	on: function(eventName, callback) {
		var self = this;
		
		self.addEvent(eventName, callback);
	},
	
	stop: function() {
		logger.info("Stopping Redis server ...");
		
		async.series([
			function(callback) {
				exec('redis-cli -h ' . self.options.redis.config.bind + ' -p ' . self.options.redis.config.port + ' shutdown', function() {
					callback(null);
				}, 500);
			}
		], function(err, results) {
			process.exit();
		});
	},
	
	initRedisConfig: function(callback) {
		var self = this;
		
		// Create the redis folder if not exists
		if(!fs.existsSync(self.options.redis.folder)) {
			fs.mkdirSync(self.options.redis.folder);
		}
		
		// Generate content of the new configuration file
		var redisConfigFileContent = '# Redis configuration file\n# Autogenerated by node-redis-embed.\n';
		for(var attr in self.options.redis.config) {
			redisConfigFileContent += '\n' + attr + ' ' + self.options.redis.config[attr];
		}
		redisConfigFileContent += '\n';
		
		// Writing base configuration into file
		fs.writeFile(self.options.redis.folder + '/redis.conf', redisConfigFileContent, function (err) {
			if (err) callback(err);
		});
		
		callback(null);
	},
	
	startRedis: function(callback) {
		var self = this;
		
		var redisOk = false;
		
		logger.debug('Starting Redis Server ...');
		self.redisServer = pty.spawn('redis-server', ['redis.conf'], {
			cwd: 'redis',
			env: process.env
		});
		
		self.redisServer.on('data', function(data) {
			if(data.indexOf('The server is now ready to accept connections') > -1) {
				logger.debug('Redis Server is now ready to accept connections.');
				
				redisOk = true;
				callback(null);
			}
			
			// Debbuging redis
			if(self.options.redis.debug) {
				console.log(data);
			}
		});
		
		setTimeout(function() {
			if(redisOk === false) {
				callback('Could not start redis !');
			}
		}, 2000);
	},
	
	createRedisClient: function(callback) {
		var self = this;
		
		self.client = redis.createClient(self.options.redis.config.port, self.options.redis.config.bind);
		self.client.select(0, function() {
			callback(null);
		});
		
		self.client.on('error', function (err) {
			callback('Redis client error ' + err);
		});
	},
	
	startRedisWeb: function(callback) {
		var self = this;
		
		logger.info('Redis web panel is listening at port : ' + self.options.web.port);
		self.redisCommander = pty.spawn('redis-commander', ['--redis-port', self.options.redis.config.port, '--redis-host', self.options.redis.config.bind, '--port', self.options.web.port, '--redis-db', 0], {
			cwd: 'redis',
			env: process.env
		});
		
		callback(null);
	},
});